package cn.com.infosec.math.ec;

import java.math.BigInteger;
import java.util.Random;

public abstract class ECCurve {
   ECFieldElement a;
   ECFieldElement b;

   public abstract int getFieldSize();

   public abstract ECFieldElement fromBigInteger(BigInteger var1);

   public abstract ECPoint createPoint(BigInteger var1, BigInteger var2, boolean var3);

   public abstract ECPoint decodePoint(byte[] var1);

   public abstract ECPoint getInfinity();

   public ECFieldElement getA() {
      return this.a;
   }

   public ECFieldElement getB() {
      return this.b;
   }

   public static class F2m extends ECCurve {
      private int m;
      private int k1;
      private int k2;
      private int k3;
      private BigInteger n;
      private BigInteger h;
      private ECPoint.F2m infinity;
      private byte mu;
      private BigInteger[] si;

      public F2m(int m, int k, BigInteger a, BigInteger b) {
         this(m, k, 0, 0, a, b, (BigInteger)null, (BigInteger)null);
      }

      public F2m(int m, int k, BigInteger a, BigInteger b, BigInteger n, BigInteger h) {
         this(m, k, 0, 0, a, b, n, h);
      }

      public F2m(int m, int k1, int k2, int k3, BigInteger a, BigInteger b) {
         this(m, k1, k2, k3, a, b, (BigInteger)null, (BigInteger)null);
      }

      public F2m(int m, int k1, int k2, int k3, BigInteger a, BigInteger b, BigInteger n, BigInteger h) {
         this.mu = 0;
         this.si = null;
         this.m = m;
         this.k1 = k1;
         this.k2 = k2;
         this.k3 = k3;
         this.n = n;
         this.h = h;
         if (k1 != 0) {
            if (k2 == 0) {
               if (k3 != 0) {
                  throw new IllegalArgumentException("k3 must be 0 if k2 == 0");
               }
            } else {
               if (k2 <= k1) {
                  throw new IllegalArgumentException("k2 must be > k1");
               }

               if (k3 <= k2) {
                  throw new IllegalArgumentException("k3 must be > k2");
               }
            }

            super.a = this.fromBigInteger(a);
            super.b = this.fromBigInteger(b);
            this.infinity = new ECPoint.F2m(this, (ECFieldElement)null, (ECFieldElement)null);
         } else {
            throw new IllegalArgumentException("k1 must be > 0");
         }
      }

      private ECPoint decompressPoint(byte[] xEnc, int ypBit) {
         ECFieldElement.F2m var3;
         ECFieldElement.F2m var10000 = var3 = new ECFieldElement.F2m;
         int var4 = this.m;
         int var5 = this.k1;
         int var6 = this.k2;
         int var7 = this.k3;
         BigInteger var11;
         var11 = new BigInteger.<init>(1, xEnc);
         var10000.<init>(var4, var5, var6, var7, var11);
         Object xEnc1;
         if (var10000.toBigInteger().equals(ECConstants.ZERO)) {
            xEnc1 = (ECFieldElement.F2m)super.b;

            for(ypBit = 0; ypBit < this.m - 1; ++ypBit) {
               xEnc1 = ((ECFieldElement)xEnc1).square();
            }
         } else {
            ECFieldElement xEnc2;
            if ((xEnc2 = this.solveQuadradicEquation(var3.add(super.a).add(super.b.multiply(var3.square().invert())))) == null) {
               throw new RuntimeException("Invalid point compression");
            }

            if (xEnc2.toBigInteger().testBit(0) != ypBit) {
               ECFieldElement.F2m xEnc3;
               ECFieldElement.F2m var10001 = xEnc3 = new ECFieldElement.F2m;
               ypBit = this.m;
               var4 = this.k1;
               var5 = this.k2;
               var6 = this.k3;
               BigInteger var12 = ECConstants.ONE;
               var10001.<init>(ypBit, var4, var5, var6, var12);
               xEnc2 = xEnc2.add(xEnc3);
            }

            xEnc1 = var3.multiply(xEnc2);
         }

         return new ECPoint.F2m(this, var3, (ECFieldElement)xEnc1);
      }

      private ECFieldElement solveQuadradicEquation(ECFieldElement beta) {
         ECFieldElement.F2m var2;
         ECFieldElement.F2m var10001 = var2 = new ECFieldElement.F2m;
         int var3 = this.m;
         int var4 = this.k1;
         int var5 = this.k2;
         int var6 = this.k3;
         BigInteger var9 = ECConstants.ZERO;
         var10001.<init>(var3, var4, var5, var6, var9);
         if (beta.toBigInteger().equals(var9)) {
            return var2;
         } else {
            Random var10;
            var10 = new Random.<init>();

            BigInteger var12;
            Object var15;
            do {
               ECFieldElement.F2m var11;
               ECFieldElement.F2m var10000 = var11 = new ECFieldElement.F2m;
               var5 = this.m;
               var6 = this.k1;
               int var7 = this.k2;
               int var8 = this.k3;
               BigInteger var13;
               var13 = new BigInteger.<init>(this.m, var10);
               var10000.<init>(var5, var6, var7, var8, var13);
               var5 = 1;
               var15 = var2;

               ECFieldElement var14;
               ECFieldElement var16;
               for(var14 = beta; var5 <= this.m - 1; var14 = var16) {
                  ECFieldElement var17 = var14 = var14.square();
                  var14 = ((ECFieldElement)var15).square().add(var14.multiply(var11));
                  var16 = var17.add(beta);
                  ++var5;
                  var15 = var14;
               }

               if (!var14.toBigInteger().equals(var12 = ECConstants.ZERO)) {
                  return null;
               }
            } while(((ECFieldElement)var15).square().add((ECFieldElement)var15).toBigInteger().equals(var12));

            return (ECFieldElement)var15;
         }
      }

      public int getFieldSize() {
         return this.m;
      }

      public ECFieldElement fromBigInteger(BigInteger x) {
         F2m var10002 = this;
         F2m var10003 = this;
         F2m var10004 = this;
         int var5 = this.m;
         int var2 = var10004.k1;
         int var3 = var10003.k2;
         int var4 = var10002.k3;
         return new ECFieldElement.F2m(var5, var2, var3, var4, x);
      }

      public ECPoint createPoint(BigInteger x, BigInteger y, boolean withCompression) {
         ECFieldElement x1 = this.fromBigInteger(x);
         ECFieldElement y1 = this.fromBigInteger(y);
         return new ECPoint.F2m(this, x1, y1, withCompression);
      }

      public ECPoint decodePoint(byte[] encoded) {
         Object var13;
         byte[] var10000;
         switch (encoded[0]) {
            case 0:
               if (encoded.length > 1) {
                  throw new RuntimeException("Invalid point encoding");
               }

               var13 = this.getInfinity();
               break;
            case 1:
            case 5:
            default:
               throw new RuntimeException("Invalid point encoding 0x" + Integer.toString(encoded[0], 16));
            case 2:
            case 3:
               var10000 = encoded;
               byte[] var19 = encoded;
               int encoded2;
               byte[] var14 = new byte[encoded2 = encoded.length - 1];
               System.arraycopy(var19, 1, var14, 0, encoded2);
               if (var10000[0] == 2) {
                  var13 = this.decompressPoint(var14, 0);
               } else {
                  var13 = this.decompressPoint(var14, 1);
               }
               break;
            case 4:
            case 6:
            case 7:
               var10000 = encoded;
               int var2;
               int var10001 = var2 = (encoded.length - 1) / 2;
               byte[] var10002 = encoded;
               byte[] var10003 = encoded;
               encoded = new byte[var2];
               int var3;
               byte[] var4 = new byte[var3 = (var10003.length - 1) / 2];
               System.arraycopy(var10002, 1, encoded, 0, var2);
               System.arraycopy(var10000, var10001 + 1, var4, 0, var3);
               Object var17 = var13 = new ECPoint.F2m;
               F2m var18 = this;
               ECFieldElement.F2m var15;
               ECFieldElement.F2m var20 = var15 = new ECFieldElement.F2m;
               int var5 = this.m;
               int var6 = this.k1;
               int var7 = this.k2;
               int var8 = this.k3;
               int var21 = var5;
               BigInteger var16;
               var16 = new BigInteger.<init>(1, encoded);
               var20.<init>(var21, var6, var7, var8, var16);
               ECFieldElement.F2m encoded1;
               var20 = encoded1 = new ECFieldElement.F2m;
               F2m var22 = this;
               F2m var10004 = this;
               F2m var10005 = this;
               int var9 = this.m;
               var5 = var10005.k1;
               var6 = var10004.k2;
               var7 = var22.k3;
               var21 = var9;
               BigInteger var10;
               var10 = new BigInteger.<init>(1, var4);
               var20.<init>(var21, var5, var6, var7, var10);
               var17.<init>(var18, var15, encoded1, false);
         }

         return (ECPoint)var13;
      }

      public ECPoint getInfinity() {
         return this.infinity;
      }

      public boolean isKoblitz() {
         return this.n != null && this.h != null && (super.a.toBigInteger().equals(ECConstants.ZERO) || super.a.toBigInteger().equals(ECConstants.ONE)) && super.b.toBigInteger().equals(ECConstants.ONE);
      }

      synchronized byte getMu() {
         if (this.mu == 0) {
            this.mu = Tnaf.getMu(this);
         }

         return this.mu;
      }

      synchronized BigInteger[] getSi() {
         if (this.si == null) {
            this.si = Tnaf.getSi(this);
         }

         return this.si;
      }

      public boolean equals(Object anObject) {
         if (anObject == this) {
            return true;
         } else if (!(anObject instanceof F2m)) {
            return false;
         } else {
            F2m anObject1 = (F2m)anObject;
            return this.m == anObject1.m && this.k1 == anObject1.k1 && this.k2 == anObject1.k2 && this.k3 == anObject1.k3 && super.a.equals(anObject1.a) && super.b.equals(anObject1.b);
         }
      }

      public int hashCode() {
         return super.a.hashCode() ^ super.b.hashCode() ^ this.m ^ this.k1 ^ this.k2 ^ this.k3;
      }

      public int getM() {
         return this.m;
      }

      public boolean isTrinomial() {
         return this.k2 == 0 && this.k3 == 0;
      }

      public int getK1() {
         return this.k1;
      }

      public int getK2() {
         return this.k2;
      }

      public int getK3() {
         return this.k3;
      }

      public BigInteger getN() {
         return this.n;
      }

      public BigInteger getH() {
         return this.h;
      }
   }

   public static class Fp extends ECCurve {
      BigInteger q;
      ECPoint.Fp infinity;

      public Fp(BigInteger q, BigInteger a, BigInteger b) {
         this.q = q;
         super.a = this.fromBigInteger(a);
         super.b = this.fromBigInteger(b);
         this.infinity = new ECPoint.Fp(this, (ECFieldElement)null, (ECFieldElement)null);
      }

      public BigInteger getQ() {
         return this.q;
      }

      public int getFieldSize() {
         return this.q.bitLength();
      }

      public ECFieldElement fromBigInteger(BigInteger x) {
         return new ECFieldElement.Fp(this.q, x);
      }

      public ECPoint createPoint(BigInteger x, BigInteger y, boolean withCompression) {
         ECFieldElement x1 = this.fromBigInteger(x);
         ECFieldElement y1 = this.fromBigInteger(y);
         return new ECPoint.Fp(this, x1, y1, withCompression);
      }

      public ECPoint decodePoint(byte[] encoded) {
         int var2;
         Object encoded2;
         byte[] var10000;
         ECFieldElement.Fp var16;
         switch (encoded[0]) {
            case 0:
               if (encoded.length > 1) {
                  throw new RuntimeException("Invalid point encoding");
               }

               encoded2 = this.getInfinity();
               break;
            case 1:
            case 5:
            default:
               throw new RuntimeException("Invalid point encoding 0x" + Integer.toString(encoded[0], 16));
            case 2:
            case 3:
               var10000 = encoded;
               byte[] var14 = encoded;
               int encoded3 = encoded[0] & 1;
               byte[] var10 = new byte[var2 = var14.length - 1];
               System.arraycopy(var10000, 1, var10, 0, var2);
               ECFieldElement.Fp var9;
               ECFieldElement.Fp var13 = var9 = new ECFieldElement.Fp;
               var13.<init>(this.q, new BigInteger(1, var10));
               ECFieldElement var11;
               if ((var11 = var13.multiply(var13.square().add(super.a)).add(super.b).sqrt()) == null) {
                  throw new RuntimeException("Invalid point compression");
               }

               if (var11.toBigInteger().testBit(0) == encoded3) {
                  encoded2 = new ECPoint.Fp.<init>(this, var9, var11, true);
               } else {
                  Object var15 = encoded2 = new ECPoint.Fp;
                  var16 = var9;
                  ECFieldElement.Fp var17 = var9 = new ECFieldElement.Fp;
                  BigInteger var10004 = this.q;
                  var17.<init>(var10004, var10004.subtract(var11.toBigInteger()));
                  var15.<init>(this, var16, var9, true);
               }
               break;
            case 4:
            case 6:
            case 7:
               var10000 = encoded;
               int var10001 = var2 = (encoded.length - 1) / 2;
               byte[] var10002 = encoded;
               byte[] var10003 = encoded;
               encoded = new byte[var2];
               int var3;
               byte[] var4 = new byte[var3 = (var10003.length - 1) / 2];
               System.arraycopy(var10002, 1, encoded, 0, var2);
               System.arraycopy(var10000, var10001 + 1, var4, 0, var3);
               ECPoint.Fp var8;
               ECPoint.Fp var12 = var8 = new ECPoint.Fp;
               var16 = new ECFieldElement.Fp(this.q, new BigInteger(1, encoded));
               ECFieldElement.Fp encoded1;
               encoded1 = new ECFieldElement.Fp.<init>(this.q, new BigInteger(1, var4));
               var12.<init>(this, var16, encoded1);
               encoded2 = var8;
         }

         return (ECPoint)encoded2;
      }

      public ECPoint getInfinity() {
         return this.infinity;
      }

      public boolean equals(Object anObject) {
         if (anObject == this) {
            return true;
         } else if (!(anObject instanceof Fp)) {
            return false;
         } else {
            Fp anObject1 = (Fp)anObject;
            return this.q.equals(anObject1.q) && super.a.equals(anObject1.a) && super.b.equals(anObject1.b);
         }
      }

      public int hashCode() {
         return super.a.hashCode() ^ super.b.hashCode() ^ this.q.hashCode();
      }
   }
}
